/*   $Id$      */
#ifdef PLATFORM_SOLARIS
#include <sys/flock.h>
#endif
#include <fcntl.h>
#include <sys/file.h>  	// LOCK_EX, LOCK_NB, flock(), LOCK_SH, LOCK_UN 
#include <sys/stat.h>   // struct stat
#include <stdlib.h>	// free(), calloc()
#include <stdio.h>	// FILE *, fopen(), fileno(), ftell(), fseek(), fflush(), fclose() 
#include <string.h>	// strlen(), strncpy()
#include <unistd.h>	// read(), write()
#include "defaults.h"   // ALL DEFAULT DEFINES

#include <sys/types.h>
#include <utime.h>
#include <sys/time.h>
#include <pcap.h>
#include "platform.h"


/**************************************************************************
 ** Security Analyst Network Connection Profiler [sancp] - A TCP/IP statistical/collection tool
 * ************************************************************************
 * * Copyright (C) 2003-2007 John Curry <john.curry@metre.net>
 * *
 * * This program is distributed under the terms of version 1.0 of the
 * * Q Public License.  See LICENSE.QPL for further details.
 * *
 * * This program is distributed in the hope that it will be useful,
 * * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * *
 * ***********************************************************************/

//#define DEBUGFH 1

#define FILEHANDLE_H
#define READ_MODE 0
#define WRITE_MODE 1
#define APPEND_MODE 2

#ifndef PLATFORM_BSD

#define LOCK_SH         1       /* shared lock */
#define LOCK_EX         2       /* exclusive lock */
#define LOCK_NB         4       /* or'd with one of the above to prevent
                                   blocking */
#define LOCK_UN         8       /* remove lock */

#endif

extern int flock (int __fd, int __operation);

class fileHandle {

	public: 
		fileHandle();

		fileHandle(const char *newfilename);

		fileHandle(FILE *OFH, int);
		
		fileHandle(const char *newfilename, int);

		fileHandle(const char *newfilename, int, char *);
		
		void setFileName(const char *newfilename);

		char * getFileName();

		int isOpen();

		int open();
		
		void reopen();

		FILE *getFileHandle();

		char *getFileType();

		void setFileType(const char *newfiletype);

		ssize_t getSize();

		ssize_t write(const char *data, int len);
		
		int read(char *data, int len);
		
		long tell();
		
		int seek(long offset, int whence);
		
		void setMode(int mode);
		
		long getStartPos();

		long getEndPos();

		long getfStartPos();

		long getfEndPos();

		int getMode();

		u_int64_t getUsers();
		
		void setSessionid(time_t sessionid);

		time_t getSessionid();

		fileHandle * attach();
		
		void detach();
		
		void close();

		void destroy();  // Do not call if object is declared on the stack - we need to change this...

		~fileHandle();

	protected:
	 	long long startpos;
		long long stoppos;
		ssize_t size;
		u_int64_t in_use;// Count of 
		FILE *handle;	// Where we write data to the file
		char *filename; // self-explanatory
		char *filetype; // name of this output type (usu. first part of filename)
		int  mode;	// are we opened for reading, writing, append...
		// temp pcap issue fix for testing 	
                //pcap_t  *ipcapt;
                //pcap_dumper_t  *opcapdt;
		time_t sessionid;
		ssize_t maxfilesize;


	private:
};

	
