/*   $Id$      */

#include "outputFileHandle.h"

/**************************************************************************
 ** Security Analyst Network Connection Profiler [sancp] - A TCP/IP statistical/collection tool
 * ************************************************************************
 * * Copyright (C) 2003-2007 John Curry <john.curry@metre.net>
 * *
 * * This program is distributed under the terms of version 1.0 of the
 * * Q Public License.  See LICENSE.QPL for further details.
 * *
 * * This program is distributed in the hope that it will be useful,
 * * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * *
 * ***********************************************************************/

outputFileHandle::outputFileHandle() : fmt(0),fmtlen(0),delimiter(0)
{ 

}
// Use this for stdout file handles
outputFileHandle::outputFileHandle(FILE *OFH, int mode = APPEND_MODE,char *ifmt=0, int len=0) :fmt(0),fmtlen(0),delimiter(DEFAULT_DELIMITER),eor(DEFAULT_EOR),wildchar(DEFAULT_NONPRINTABLE)
{ 
	::fileHandle(OFH,mode);
	setFormat(ifmt,len);
}

outputFileHandle::outputFileHandle(const char *newfilename,char *ifmt=0, int len=0, int mode=APPEND_MODE, char *newfiletype=NULL ) : fmt(0),fmtlen(0),delimiter(DEFAULT_DELIMITER),eor(DEFAULT_EOR),wildchar(DEFAULT_NONPRINTABLE)
{
	setMode(mode);
	setFormat(ifmt,len);
	setFileName(newfilename);
	setFileType(newfiletype);
} 


int outputFileHandle::getFormatLen() 
{
	return fmtlen;
}

char * outputFileHandle::getFormat() 
{
	return fmt;
}

char * outputFileHandle::setFormat(char *ifmt, int len) 
{
	if(fmt) { free(fmt); fmt=0; }
	fmtlen=len;
	if((fmt=(char *) calloc(len,1))!=0)
		memcpy(fmt,ifmt,len);
	return fmt;
}


void outputFileHandle::printFormat(char *LOG, int len, char **tfmtnames,int MAXFLDS,int MAXFLDSIZE){

        char ttext[len+1];
	bzero(ttext,len+1);
	int tsize;
	char *tdelimiter=0;
	char *twildchar=0;

	if(filetype){

          sprintf(ttext,"format %s ",filetype);
	  tsize=strlen(ttext); 

          if(delimiter){

              switch (delimiter){
                case ' ':
		tdelimiter="\\s";
                break;
                case '\t':
		tdelimiter="\\t";
                break;
                case '\n':
		tdelimiter="\\n";
                break;
              }

	      if(tdelimiter){
       	        sprintf(ttext+tsize,"delimiter=%s ",tdelimiter);
	      }else{
 	        sprintf(ttext+tsize,"delimiter=%c ",delimiter);
	      }

	     tsize=strlen(ttext); 

	  }

          if(wildchar){

              switch (wildchar){
                case ' ':
                twildchar="\\s";
                break;
                case '\t':
                twildchar="\\t";
                break;
                case '\n':
                twildchar="\\n";
                break;
              }

              if(twildchar){
                sprintf(ttext+tsize,"wildchar=%s ",twildchar);
              }else{
                sprintf(ttext+tsize,"wildchar=%c ",wildchar);
              }

             tsize=strlen(ttext);

          }


          char *tmpfldnames=fmt;
	  int ttsize=0;
	  char tttext[len+1];
   	  bzero(tttext,len+1);
	  int tfmtval=0;

          while((tsize+ttsize<len)&&(tfmtval<fmtlen)){

   	     bzero(tttext,len);

             /* keep track of the current index of tfmtnames in tfmtval */
             tmpfldnames+=MAXFLDSIZE; 
             sprintf(tttext,"%s",(char *)tfmtnames+(fmt[tfmtval]*MAXFLDSIZE));
	     ttsize=strlen(tttext); 
	     tfmtval++;

	     /* check if we have enough space to add to the LOG line */
	     if((tsize+ttsize+2)<len){
               sprintf(ttext+tsize,"%s ",tttext);
	       tsize=strlen(ttext); 
	     }	
          }
          snprintf(LOG,(tsize<len)?tsize:(len-1),"%s\n",ttext);

        }else{
             sprintf(LOG,"format unknown");
	}
}


char outputFileHandle::getWildchar() 
{
	return wildchar;
}

void outputFileHandle::setWildchar(char c) 
{
	wildchar=c;
}

char outputFileHandle::getEor() 
{
	return eor;
}

void outputFileHandle::setEor(char c) 
{
	eor=c;
}

char outputFileHandle::getDelimiter() 
{
	return delimiter;
}

void outputFileHandle::setDelimiter(char c) 
{
	delimiter=c;
}

outputFileHandle * outputFileHandle::attach() 
{
	fileHandle::attach();
	return this;
}

outputFileHandle::~outputFileHandle() 
{ 
	if(fmt) { free(fmt); fmt=0; }
}


